pub mod consumers;
pub mod handler;
pub mod staging;
pub mod ingest {
    use crate::state::SharedState;

    use super::{
        consumers::{Consumer, Consumers},
        staging::{Staging, StagingEvent},
    };

    #[derive(Clone)]
    pub struct Ingest {
        staging: Staging,
        consumers: Consumers,
    }

    impl Ingest {
        pub fn new(staging: Staging, consumers: Consumers) -> Self {
            Self { staging, consumers }
        }

        pub async fn publish(&self, event: impl Into<StagingEvent>) -> anyhow::Result<()> {
            let event: StagingEvent = event.into();
            let topic = event.topic.clone();
            let key = {
                if event.key.is_empty() {
                    None
                } else {
                    Some(event.key.clone())
                }
            };

            let offset = self.staging.publish(event).await?;
            self.consumers.notify_update(topic, key, offset).await?;

            Ok(())
        }
    }

    pub trait IngestState {
        fn ingest(&self) -> Ingest;
    }

    impl IngestState for SharedState {
        fn ingest(&self) -> Ingest {
            Ingest::new(self.staging.clone(), self.consumers.clone())
        }
    }
}
